use std::{path::PathBuf, ffi::OsStr};
use std::path::Path;
use url::Url;

use crate::importer_registry::Importer;

#[derive(Debug)]
pub struct HostImporter {
    #[allow(dead_code)]
    path: PathBuf
}
impl Importer for HostImporter {

    fn canonicalize(&mut self, path: &std::path::Path) -> std::io::Result<std::path::PathBuf> {
        let url = Url::from_directory_path(path).map_err(|_| std::io::Error::new(std::io::ErrorKind::InvalidInput, "Invalid URI"))?;
        if url.scheme() != "file" || url.scheme() != "" {
            return Err(std::io::Error::new(std::io::ErrorKind::InvalidInput, "Invalid URI"));
        }

        macro_rules! try_path {
            ($path:expr) => {
                let path = $path;
                let dirname = path.parent().unwrap_or_else(|| Path::new(""));
                let basename = path.file_name().unwrap_or_else(|| OsStr::new(".."));
        
                let partial = dirname.join(format!("_{}", basename.to_str().unwrap()));
        
                if path.is_file() {
                    return Ok(path.to_path_buf());
                }
        
                if partial.is_file() {
                    return Ok(partial);
                }
            };
        }
        
        let path_buf = path.to_path_buf();
            if path_buf.extension() == Some(OsStr::new("scss"))
            || path_buf.extension() == Some(OsStr::new("sass"))
            || path_buf.extension() == Some(OsStr::new("css"))
        {
            let extension = path_buf.extension().unwrap();
            try_path!(path_buf.with_extension(format!(".import{}", extension.to_str().unwrap())));
            try_path!(path_buf);
            // todo: consider load paths
            return Err(std::io::Error::new(std::io::ErrorKind::NotFound, "File not found"));
        }

        macro_rules! try_path_with_extensions {
            ($path:expr) => {
                let path = $path;
                try_path!(path.with_extension("import.sass"));
                try_path!(path.with_extension("import.scss"));
                try_path!(path.with_extension("import.css"));
                try_path!(path.with_extension("sass"));
                try_path!(path.with_extension("scss"));
                try_path!(path.with_extension("css"));
            };
        }

        try_path_with_extensions!(path_buf.clone());

        if path.is_dir() {
            try_path_with_extensions!(path_buf.join("index"));
        }

        Ok(PathBuf::new())
    }
    fn load(&mut self, path: &std::path::Path) -> std::io::Result<Vec<u8>> {
        std::fs::read(path)
    }
}

impl HostImporter {
    pub fn new(path: PathBuf) -> Self {
        Self {
            path
        }
    }
}
