use std::error::Error;

use clap::Parser;
use fern::Dispatch;

mod import;
mod importer_registry;
mod buf_io;
mod compiler;
#[derive(Debug, Parser)]
struct Args {
    #[clap(short, long, default_value = "error")]
    log_level: String,
    #[clap(short, long, default_value = "0")]
    wait: u64
}
fn start_logger(log_level: &str) {
    let level_filter = match log_level.to_lowercase().as_str() {
        "trace" => log::LevelFilter::Trace,
        "debug" => log::LevelFilter::Debug,
        "info" => log::LevelFilter::Info,
        "warn" => log::LevelFilter::Warn,
        "error" => log::LevelFilter::Error,
        _ => log::LevelFilter::Off,
    };

    // setup fern logger
    let colors = fern::colors::ColoredLevelConfig::new()
        .error(fern::colors::Color::Red)
        .warn(fern::colors::Color::Yellow)
        .info(fern::colors::Color::Green)
        .debug(fern::colors::Color::Magenta)
        .trace(fern::colors::Color::Cyan);
    Dispatch::new()
        .format(move |out, message, record| {
            out.finish(format_args!(
                "[{}][{}] {}",
                colors.color(record.level()),
                record.target(),
                message
            ))
        })
        .level(level_filter)
        .chain(std::io::stderr())
        // .chain(fern::log_file("output.log").unwrap()) // remove logfile
        .apply()
        .unwrap();

}
fn main() -> Result<(),Box<dyn Error>> {
    use crate::buf_io::MessageType;

    let args = Args::parse();
    start_logger(&args.log_level);

    log::info!("Starting up");

    loop {
        let event = buf_io::read_message()?;
        match event.0 {
            MessageType::CompileRequest(req) => {
                compiler::compile_request(req, event.1)?;
            }
            _ => {
                log::debug!("Something other than compile request recived, ignoring")
            }
        }
    }
}